@@ -6,32 +6,45 @@ use strict;
 use Games::Rezrov::ZProperty;
 use Games::Rezrov::ZText;
 use Games::Rezrov::Inliner;
-use Games::Rezrov::MethodMaker ([],
-			 qw(
-			    prop_addr
-			    object_id
-			    pointer_size
-			    property_start_index
-			    attrib_offset
-			    sibling_offset
-			    child_offset
-			    parent_offset
-			    property_cache
-			   ));
+use Games::Rezrov::InlinedPrivateMethod;
 
-use SelfLoader;
+my $code = Games::Rezrov::InlinedPrivateMethod->new("-manual" => 1,
+						    "-names" =>
+						    [ qw(
+							 _object_id
+							 _pointer_size
+							 _attrib_offset
+							 _property_start_index
+							 _prop_addr
+							 _parent_offset
+							 _child_offset
+							 _sibling_offset
+							 _property_cache
+							)
+						    ],
+						   );
+Games::Rezrov::Inliner::inline($code);
 
-#use Carp qw(cluck);
+#print $$code; die;
+eval $$code;
+die $@ if $@;
+
+1;
+
+__DATA__
+
+sub object_id {
+  # public, read-only
+  return $_[0]->_object_id();
+}
 
-my $INLINE_CODE = '
 sub get_ptr {
   # get a value, depending on pointer size
-  # args: offset
-  if ($_[0]->pointer_size() == 1) {
-    return GET_BYTE_AT($_[1]);
-  } else {
-    return GET_WORD_AT($_[1]);
-  }
+  # args: offset.
+  # - This seems slightly faster than using a different class
+  #   for each type size and inheriting back to ZObject.
+  return ($_[0]->_pointer_size() == 1) ?
+     GET_BYTE_AT($_[1]) : GET_WORD_AT($_[1]);
 }
 
 sub test_attr {
@@ -49,14 +62,14 @@ sub test_attr {
   my $bit_position = ($attribute % 8);
   # which bit in the byte (starting at high bit, counted as #0)
   my $bit_shift = 7 - $bit_position;
-  my $the_byte = GET_BYTE_AT($self->attrib_offset + $byte_offset);
+  my $the_byte = GET_BYTE_AT($self->_attrib_offset() + $byte_offset);
   my $the_bit = ($the_byte >> $bit_shift) & 0x01;
   # move target bit into least significant byte
 
   if (Games::Rezrov::ZOptions::SNOOP_ATTR_TEST()) {
     write_message(sprintf "[Test attribute %d of %s (%s) = %d]",
 		  $attribute,
-		  $self->object_id(),
+		  $self->_object_id(),
 		  ${$self->print()},
                   $the_bit == 1);
   }
@@ -71,14 +84,14 @@ sub set_attr {
   my $bit_position = $attribute % 8;
   my $bit_shift = 7 - $bit_position;
   my $mask = 1 << $bit_shift;
-  my $where = $self->attrib_offset() + $byte_offset;
+  my $where = $self->_attrib_offset() + $byte_offset;
   my $the_byte = GET_BYTE_AT($where);
   $the_byte |= $mask;
   Games::Rezrov::StoryFile::set_byte_at($where, $the_byte);
   if (Games::Rezrov::ZOptions::SNOOP_ATTR_SET()) {
     write_message(sprintf "[Set attribute %d of %s (%s)]",
 		  $attribute,
-		  $self->object_id(),
+		  $self->_object_id(),
 		  ${$self->print()});
   }
 }
@@ -90,14 +103,14 @@ sub clear_attr {
   my $bit_position = ($attribute % 8);
   my $bit_shift = 7 - $bit_position;
   my $mask = ~(1 << $bit_shift);
-  my $where = $self->attrib_offset() + $byte_offset;
+  my $where = $self->_attrib_offset() + $byte_offset;
   my $the_byte = GET_BYTE_AT($where);
   $the_byte &= $mask;
   Games::Rezrov::StoryFile::set_byte_at($where, $the_byte);
   if (Games::Rezrov::ZOptions::SNOOP_ATTR_CLEAR()) {
     write_message(sprintf "[Clear attribute %d of %s (%s)]",
 		  $attribute,
-		  $self->object_id(),
+		  $self->_object_id(),
 		  ${$self->print()}
                   );
   }
@@ -114,14 +127,14 @@ sub new {
 #  my $self = {};
   bless $self, $type;
 
-  $self->object_id($object_id);
+  $self->_object_id($object_id);
   my $header = Games::Rezrov::StoryFile::header();
 
   my $attrib_offset = $header->attribute_starter() +
     $header->object_bytes() * ($object_id - 1);
   
   my $pointer_size = $header->pointer_size();
-  $self->pointer_size($pointer_size);
+  $self->_pointer_size($pointer_size);
 
   my $parent_offset = $attrib_offset + $header->attribute_bytes();
   my $sibling_offset = $parent_offset + $pointer_size;
@@ -134,40 +147,26 @@ sub new {
 
   # spec section 12.4: property table header,
   # first byte is length of "short name" text
-  $self->property_start_index($prop_addr + 1 + ($text_words * 2));
-  
-  $self->prop_addr($prop_addr);
-  $self->attrib_offset($attrib_offset);
-  $self->parent_offset($parent_offset);
-  $self->sibling_offset($sibling_offset);
-  $self->child_offset($child_offset);
-  $self->property_cache({});
+  $self->_property_start_index($prop_addr + 1 + ($text_words * 2));
+  $self->_prop_addr($prop_addr);
+  $self->_attrib_offset($attrib_offset);
+  $self->_parent_offset($parent_offset);
+  $self->_sibling_offset($sibling_offset);
+  $self->_child_offset($child_offset);
+  $self->_property_cache({});
 
   return $self;
 }
 
-';
-
-Games::Rezrov::Inliner::inline(\$INLINE_CODE);
-eval $INLINE_CODE;
-undef $INLINE_CODE;
-
-
-1;
-
-__DATA__
-
-
 sub get_property {
   # arg: property number
   my $zp;
-  my $pc = $_[0]->property_cache();
-#  printf STDERR "want prop %s of obj %s (%s)...", $_[1], $_[0]->object_id(), ${$_[0]->print};
+  my $pc = $_[0]->_property_cache();
   if ($zp = $pc->{$_[1]}) {
 #    printf STDERR "cache hit\n";
     return $zp;
   } else {
-    $zp = new Games::Rezrov::ZProperty($_[1], $_[0]);
+    $zp = new Games::Rezrov::ZProperty($_[1], $_[0], $_[0]->_property_start_index());
     $pc->{$_[1]} = $zp;
 #    print STDERR "\n";
     return $zp;
@@ -182,7 +181,7 @@ sub remove {
     # no parent, quit
     return;
   } else {
-    my $object_id = $self->object_id;
+    my $object_id = $self->_object_id();
     my $child = $parent->get_child();
     # get child of old parent
     if ($parent->get_child_id() == $object_id) {
@@ -194,7 +193,7 @@ sub remove {
       my $this_sib;
       for ($this_sib = $child->get_sibling(); defined($this_sib);
 	   $prev_sib = $this_sib, $this_sib = $this_sib->get_sibling()) {
-	if ($this_sib->object_id() == $object_id) {
+	if ($this_sib->_object_id() == $object_id) {
 	  # found it
 	  $prev_sib->set_sibling_id($this_sib->get_sibling_id());
 	  last;
@@ -214,15 +213,15 @@ sub remove {
 }
 
 sub get_parent {
-  return $_[0]->get_object($_[0]->parent_offset());
+  return $_[0]->get_object($_[0]->_parent_offset());
 }
 
 sub get_sibling {
-  return $_[0]->get_object($_[0]->sibling_offset());
+  return $_[0]->get_object($_[0]->_sibling_offset());
 }
 
 sub get_child {
-  return $_[0]->get_object($_[0]->child_offset());
+  return $_[0]->get_object($_[0]->_child_offset());
 }
 
 sub get_object {
@@ -245,7 +244,7 @@ sub get_object {
 sub set_ptr {
   # args: offset value
   # set a value, depending on pointer size
-  if ($_[0]->pointer_size() == 1) {
+  if ($_[0]->_pointer_size() == 1) {
     Games::Rezrov::StoryFile::set_byte_at($_[1], $_[2]);
   } else {
     Games::Rezrov::StoryFile::set_word_at($_[1], $_[2]);
@@ -253,35 +252,35 @@ sub set_ptr {
 }
 
 sub get_child_id {
-  return $_[0]->get_ptr($_[0]->child_offset());
+  return $_[0]->get_ptr($_[0]->_child_offset());
 }
 
 sub get_parent_id {
-  return $_[0]->get_ptr($_[0]->parent_offset());
+  return $_[0]->get_ptr($_[0]->_parent_offset());
 }
 
 sub get_sibling_id {
-  return $_[0]->get_ptr($_[0]->sibling_offset());
+  return $_[0]->get_ptr($_[0]->_sibling_offset());
 }
 
 sub set_parent_id {
   # set parent object id of this object to specified id
-  $_[0]->set_ptr($_[0]->parent_offset(), $_[1]);
+  $_[0]->set_ptr($_[0]->_parent_offset(), $_[1]);
 }
 
 sub set_child_id {
-  $_[0]->set_ptr($_[0]->child_offset(), $_[1]);
+  $_[0]->set_ptr($_[0]->_child_offset(), $_[1]);
 }
 
 sub set_sibling_id {
-  $_[0]->set_ptr($_[0]->sibling_offset(), $_[1]);
+  $_[0]->set_ptr($_[0]->_sibling_offset(), $_[1]);
 }
 
 sub print {
   my ($self, $text) = @_;
   $text = new Games::Rezrov::ZText() unless $text;
   # eek
-  return scalar $text->decode_text($self->prop_addr() + 1);
+  return scalar $text->decode_text($self->_prop_addr() + 1);
 }
 
 sub write_message {
